package com.ibm.sysmgt.raidmgr.cim.provider.event;

import java.io.*;
import java.util.*;
import java.rmi.RemoteException;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.cim.provider.*;

/**
*  This object polls the Agent for events.  This is only necessary in the IPC case, since
*  in the Local and RMI case, the agent will call the GUIAlertListener directly with 
*  events as they are generated.
*/
class EventPollingThread extends Thread implements Constants
{
   private RAIDEventProvider eventProvider;
   private PrintWriter log;
   private static long SLEEP_TIME = 10000; 
   private static long INITIAL_SLEEP_TIME = 30000; 
   private boolean theThreadShouldContinue = true;
   private int overallStatus = UNKNOWN;
   private int newOverallStatus = UNKNOWN;   
   private boolean recievedInitialEvents = false;
   private boolean remoteExceptionLogged = false;
   private final static char OS = JCRMOS.getOS();
   private boolean adapterPresent = true;

   /**
   *  Create an object to poll the agent for events.
   */
   public EventPollingThread(RAIDEventProvider eventProvider, PrintWriter log)
   {
      this.eventProvider = eventProvider;
      this.log = log;      
   }

   /**
   *  Continuosly poll the agent for threads until told to stop.
   */
   public void run()
   {
      Vector events = new Vector();
      while (theThreadShouldContinue)
      {         
         // If the event provider starts before the Director agent then we may miss the
         // initial events.  In that case try to get them again.
         if (recievedInitialEvents == false)
         {            
            Sleep(INITIAL_SLEEP_TIME);
            getInitialEvents();            
            continue;
         }
         
         try
         {
            // Get Events
            events = eventProvider.getAgent().getEvents();

            // Get Overall Status
            newOverallStatus = eventProvider.getAgent().getCachedConfigAllOverallStatus();
            remoteExceptionLogged = false;            
         }
         catch (RemoteException ex)
         {
            logRemoteException("(EventPollingThread) Waiting to communicate with ServeRAID agent", ex);

            // Change the cached config policy to POLLING if failed communication
            eventProvider.setCachedConfigRefreshPolicy(RAIDProvider.POLLING);

            // If we can't communicate with the agent we don't want to suck bandwidth trying
            Sleep(INITIAL_SLEEP_TIME);
            continue;
         }

         // If we can get events, change cache update policy to EVENT_DRIVEN
         eventProvider.setCachedConfigRefreshPolicy(RAIDProvider.EVENT_DRIVEN);

         // Send events, and system health update if needed.
         if (events != null)
         {
            // Notify instance providers to update the cached config
            eventProvider.setCachedConfigDirty(true);

            for (Enumeration e = events.elements(); e.hasMoreElements();)
               filterEvent((RaidEvent)e.nextElement());
         }         
         checkOverallStatus();
         Sleep(SLEEP_TIME);
      }

      // If exiting the event thread, change cache policy to POLLING
      eventProvider.setCachedConfigRefreshPolicy(RAIDProvider.POLLING);
   }

   /**
   *  Stop the polling thread.
   */
   void endThread()
   {
      theThreadShouldContinue = false;
   }

   /**
     * This method gets the initial set of events and sends them to the 
     * registered listener
     */
   public void getInitialEvents()
   {
      try
      {
         // Initialize the overall status
         newOverallStatus = eventProvider.getAgent().getCachedConfigAllOverallStatus();

         // Send the initial events to the listener
         for (Enumeration e = eventProvider.getAgent().getInitialEvents().elements(); e.hasMoreElements();)
            filterEvent((RaidEvent)e.nextElement());         
      }
      catch (RemoteException ex)
      {      
         logRemoteException("(EventPollingThread) Waiting for initial set of events", ex);
         recievedInitialEvents = false;
         return;
      }

      checkOverallStatus();

      recievedInitialEvents = true;
      remoteExceptionLogged = false;
   }

   /**
     * This method filters each event, throws away all internal events, and generates a new
     * event if the overall system status has changed. The events are then forwarded to the
     * event listener.
     * @param the RaidEvent to filter
     */
   private void filterEvent(RaidEvent evt)
   {
      // Don't send a health event if there are no controllers, because in reality there is nothing to manage
      if (evt.getEventCode() == Constants.NO_CONTROLLERS)
         adapterPresent = false;
            
      // Detect a hot add
      if (evt.getEventCode() == Constants.CONTROLLER_ADDED)          
         adapterPresent = true;

      // Throw away internal and refresh events
      if ((evt.getEventType() != INTERNAL_EVENT) && (evt.getEventType() != REFRESH_EVENT))
      {  
         sendOverallEvent(evt);
         sendEvent(evt);
      }
   }

   /**
     * This method checks for a state change from critical / fatal to normal.  This is
     * implemented to support system health since system health requires notificaion
     * of "normal" events.
     */
   private void checkOverallStatus()
   {
      // See if the overall status has changed to normal
      if (overallStatus != newOverallStatus)
      {         
         if (newOverallStatus == Constants.NORMAL)
         {
            try
            {
               sendOverallEvent(new RaidEvent(eventProvider.getAgent().getServerName(), Constants.OVERALL_STATUS_NORMAL_EVENT, "eventOverallStatusIsNormal", null, JCRMUtil.getNLSString("eventOverallStatusIsNormal"), 0));
            }
            catch (RemoteException ex)
            {
               ex.printStackTrace(log);
            }            
         }            
      }

      overallStatus = newOverallStatus;
   }

   /**
     * This method sends an event to the registered listener
     * @param the RaidEvent to send
     */
   private void sendEvent(RaidEvent evt)
   {
         eventProvider.processAlert(evt);
   }

   /**
     * This method sends an Overall Status event to the registered listener
     * @param the event type
     * @param the raid event generating the overall event, if one exists
     */
   private void sendOverallEvent(RaidEvent event)
   {
      // Don't send any health events if there is nothing to manage
      if (adapterPresent == false)
         return;

      RaidEvent evt = null;
      
      try
      {
         if (newOverallStatus == Constants.CRITICAL)
            evt = new RaidEvent(eventProvider.getAgent().getServerName(), Constants.OVERALL_STATUS_CRITICAL_EVENT, event.getEventKey(), event.getEventKeyArgs(), event.getEventString(), event.getAdapterNumber());
         else if (newOverallStatus == Constants.FATAL)
            evt = new RaidEvent(eventProvider.getAgent().getServerName(), Constants.OVERALL_STATUS_FATAL_EVENT, event.getEventKey(), event.getEventKeyArgs(), event.getEventString(), event.getAdapterNumber());
         else
            evt = new RaidEvent(eventProvider.getAgent().getServerName(), Constants.OVERALL_STATUS_NORMAL_EVENT, event.getEventKey(), event.getEventKeyArgs(), event.getEventString(), event.getAdapterNumber());
      }
      catch (RemoteException ex)
      {
         ex.printStackTrace(log);
      }

      sendEvent(evt);
   }

   private void Sleep(long milliSeconds)
   {
      try
      {
         Thread.sleep( milliSeconds );
      }
      catch (InterruptedException ex)
      {
         ex.printStackTrace(log);
      }
   }

   private void logRemoteException(String message, RemoteException ex)
   {
      // Keep the log file from filling up with the same error
      if (remoteExceptionLogged == false)
      {            
         StringTokenizer st = new StringTokenizer(ex.getMessage());
         while (st.hasMoreTokens())
         {
            String exCode = st.nextToken().toLowerCase();
            if (exCode.startsWith("code"))
            {                              
               log.println(message + " (RMex " + exCode + ").");
               remoteExceptionLogged = true;
               return;
            }
         }     

         log.println(message + " (RMex).");
         remoteExceptionLogged = true;
      }
   }
}
