package com.ibm.sysmgt.raidmgr.cim.provider.v2;

import java.io.*;
import java.util.*;
import java.rmi.RemoteException;

import javax.wbem.cim.*;
import javax.wbem.client.*;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.agent.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.RaidSystem;

/** This class provides the base for all CIM providers. It holds the remote DataProc reference and 
  * allows a CIM provider to communicate with the ServeRAID agent.  
  */

public class RAIDProvider
{
   private final static String ProviderProperties = "raidprovider.pps";
   private final static String CACHE_REFRESH_INTERVAL_DEFAULT = "120000";
   protected final static char OS = JCRMOS.getOS();

   // Connection mode (Local Only, RMI, IPC)
   public final static int LOCAL_ONLY = 0;
   public final static int RMI = 1;
   public final static int IPC = 2;
   private int protocol = LOCAL_ONLY;

   private ManagementAgentIntf agent;   
   protected ProviderCIMOMHandle cimom;

   // Cached Config related items
   private static RaidSystem cachedRaidSystem;
   private static long cacheRefreshInterval; // cache refresh interval in milliseconds
   private static long cacheTimeStamp = 0;   // Last time the cache was refreshed
   public final static int EVENT_DRIVEN = 0;
   public final static int POLLING = 1;
   private static int cacheRefreshPolicy = POLLING;
   private static boolean staleCache = true; // Allows event provider to signal a dirty cache
   
   protected final String logHeader;
   public static PrintWriter log = setupLog("RAIDProvider.log");   

   static
   {
      staticInitialize();
   }

   /**
     * Constructor to create the RaidProvider 
     */
   public RAIDProvider(String logHeader)
   {            
      if (false /* !!! fix me */)
      {      
         agent = new ManagementAgentIPC(log); // Connect to Director agent
         protocol = IPC;
      }
      else
      {      
         // Local only mode
         agent = new ManagementAgent(null);   // Local only mode
         protocol = LOCAL_ONLY;
      }

      this.logHeader = logHeader;
   }

   public RAIDProvider()
   {
      this.logHeader = "";
   }

   /** Satisfies the CIMProvider interface */
   public void initialize(CIMOMHandle cimom) throws CIMException
   {
      log.println(logHeader + "initialize()");      
      this.cimom=(ProviderCIMOMHandle)cimom;
   }

   /** Satisfies the EventProvider interface */
   public void cleanup() throws CIMException
   {
      log.println(logHeader + "cleanup()");
   }

   /**
     * This method returns a reference to the data proc that can be to make
     * remote calls to the RMI agent
     * @return an object from the agent that implements the DataProcIntf
     */
   public DataProcIntf getDataProc()
   {
      return agent.getDataProcessor();    
   }   

   protected void setEventsEnabled(boolean enabled, AlertListener listener) throws CIMException
   {
      if (protocol == LOCAL_ONLY)
      {
         try
         {         
            ManagementAgent localAgent = (ManagementAgent) agent;
            NotificationManagerLocal localNM = (NotificationManagerLocal)localAgent.getNotificationManager();
            if (enabled == true)                     
               localNM.setAlertListener(listener);
            else
               localNM.removeAlertListener();
         }
         catch (ClassCastException ex)
         {
            throw new CIMProviderException(CIMProviderException.GENERAL_EXCEPTION);
         }
      }
      else if (protocol == IPC)
      {
         //!!! Fix Me
      }
      else if (protocol == RMI)
      {
         //!!! Fix Me
      }
      else
         throw new CIMProviderException(CIMProviderException.GENERAL_EXCEPTION);
   }

   public Vector getEvents() throws RemoteException
   {
      //!!! Fix Me
      return null /*agent.getEvents()*/;
   }

   public Vector getInitialEvents() throws RemoteException
   {
      //!!! Fix Me
      return null /*agent.getInitialEvents()*/;
   }

   /**
   * This method returns a RaidSystem which describes the RAID configuration
   * Note that this does a "getConfigAll" which means the RAID configuration returned
   * inclues dummy data such as empty drives and free logical drives.
   * @return the current RaidSystem
   */
   public RaidSystem getRaidSystem() throws RemoteException
   {
      if (cacheIsStale())
      {               
         log.println("(RAIDProvider) stale cache - refreshing. " + new Date());
         cachedRaidSystem = (RaidSystem)getDataProc().invokeMethod("getConfigAll", null);      
         cacheTimeStamp = System.currentTimeMillis();
         staleCache = false;
         return cachedRaidSystem;
      }
      else
      {
         log.println("(RAIDProvider) using cached RaidSystem." );
         return cachedRaidSystem;            
      }
   }

   private boolean cacheIsStale()
   {
      if (cacheRefreshPolicy == EVENT_DRIVEN)
         return staleCache;
      else
      {
         long currentTime = System.currentTimeMillis();

         if (currentTime > (cacheTimeStamp + cacheRefreshInterval))
            return true;

         return false;
      }
   }

   public void setCachedConfigRefreshPolicy(int refreshPolicy)
   {
      cacheRefreshPolicy = refreshPolicy;
   }

   public void setCachedConfigDirty(boolean isDirty)
   {
      staleCache = isDirty;
   }

   /** This method will set up a log file in the TEMPDIR directory */
   public static PrintWriter setupLog(String logFileName)
   {
      try
      {
         File logFile = new File(System.getProperty("logdir") + System.getProperty("file.separator") + logFileName);
         return new PrintWriter(new FileOutputStream(logFile), true);
      }
      catch(IOException e)
      {
         return new PrintWriter(new StringWriter());
      }
   }

   private static void staticInitialize()
   {
      Properties defaults = new Properties();
      defaults.put("cache.refresh.interval", CACHE_REFRESH_INTERVAL_DEFAULT);

      FileInputStream in = null;
      Properties properties = new Properties(defaults);

      try
      {         
         in = new FileInputStream(new File(System.getProperty("DIRECTOR_HOME") + System.getProperty("file.separator") + "data", ProviderProperties));
         properties.load(in);
      }
      catch (FileNotFoundException e)
      {
         in = null;
         log.println("Note: Properties file " + ProviderProperties + " not found, using defaults");
      }
      catch (IOException e)
      {
         JCRMUtil.ErrorLog("Can not open properties file: " + ProviderProperties);
         e.printStackTrace(log);
      }
      finally
      {
         if (in != null)
         {
            try
            {
               in.close();
            }
            catch (IOException e)
            {
            }
            in = null;
         }
      }

      try
      {
         cacheRefreshInterval = Long.parseLong(properties.getProperty("cache.refresh.interval"));
      }      
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         cacheRefreshInterval = Long.parseLong(CACHE_REFRESH_INTERVAL_DEFAULT);
      }

      log.println("Note: Cache Refresh Interval is: " + cacheRefreshInterval + " milliseconds."); 
   }
}
