package com.ibm.sysmgt.raidmgr.cim.provider.v2.event;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.lsi.*;
import com.ibm.sysmgt.raidmgr.cim.provider.v2.*;
import com.ibm.sysmgt.raidmgr.cim.provider.v2.instance.*;
import com.ibm.sysmgt.raidmgr.agent.NotificationManagerLocal;

import javax.wbem.cim.*;
import javax.wbem.provider.*;
import javax.wbem.query.SelectExp;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

/** Processes and sends events to CIM consumers via the UMS cimwmi proxy provider.
  * Events are recieved from EventPollingThread which polls the ServeRAID
  * Director agent for events and notifies this object when events are recieved
  * via the ProcessAlert method.
  */
public class RAIDEventProvider extends RAIDProvider implements AlertListener, EventProvider
{
   protected static PrintWriter eventLog = setupLog("RAIDEventProvider.log");
   
   public final static int NORMAL_EVENT = 0;   /* Information (CIM 2.6) */
   public final static int WARNING_EVENT = 3;  /* Degraded/Warning (CIM 2.6) */
   public final static int CRITICAL_EVENT = 7; /* Fatal/NonRecoverable (CIM 2.6) */
   final static String RAID_EVENT = "ADPT_StorageRAIDAlertIndication";

   public RAIDEventProvider()
   {            
      super("RAIDEventProvider:");
      eventLog.println(logHeader + "RAIDEventProvider()");      
   }

   public void authorizeFilter(SelectExp filter, String eventType, CIMObjectPath classPath, String owner) throws CIMException
   {
      eventLog.println(logHeader + "authorizeFilter()");
   }

   /** Satisfies the EventProvider interface */
   public void activateFilter(SelectExp filter, String eventType, CIMObjectPath classPath, boolean firstActivation) throws CIMException
   {
      eventLog.println(logHeader + "activateFilter() - RAID_EVENT");

      if (eventType.equalsIgnoreCase(RAID_EVENT)) 
      {      
         if (firstActivation)
         {
            eventLog.println(logHeader + "activateFilter() - Starting event provider.");
            setEventsEnabled(true, this);
         }
      }
      else
      {   
         eventLog.println(logHeader + eventType + " != " + RAID_EVENT);
         throw new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED);
      }
   }

   /** Satisfies the EventProvider interface */
   public void deActivateFilter(SelectExp filter, String eventType, CIMObjectPath classPath, boolean lastActivation) throws CIMException
   {
      eventLog.println(logHeader + "deActivateFilter()");
      if (lastActivation == true)
      {
         eventLog.println(logHeader + "deActivatateFilter() - Stopping event provider.");
         setEventsEnabled(false, this);
      }
   }

   /** Satisfies the EventProvider interface */
   public boolean mustPoll(SelectExp filter, String eventType, CIMObjectPath classPath) throws CIMException
   {
      // Not sure what this does
      eventLog.println(logHeader + "mustPoll()");
      return false;
   }

   /** Sends the event recieved in ProcessAlert to the cimwmi proxy provider
     * who in turn forwards the event to the various CIM event consumers.
     */
   void sendEvent(RaidEvent event, String eventType)
   {
      eventLog.println("sendEvent() eventType = " + eventType);                                                       
      CIMObjectPath cop = new CIMObjectPath(eventType, "\\root\\cimv2");
      try
      {          
          CIMClass indicationClass = cimom.getClass(cop, false, false, false, null);
          CIMInstance indication = indicationClass.newInstance();          
          cimom.deliverEvent(cop.getNameSpace(), mapEvent(indication, event));
      }
      catch(Exception e)
      {
          e.printStackTrace(eventLog);
      }
   }

   /** Translates a RaidEvent into a CIM event */
   CIMInstance mapEvent(CIMInstance indication, RaidEvent event)
   {            
      int eventSeverity = RAIDEventProvider.NORMAL_EVENT;
      String targetObjectPath = "";

      switch(event.getEventType())
      {
      case Constants.WARNING_EVENT:
         eventSeverity = RAIDEventProvider.WARNING_EVENT;
         targetObjectPath = getTargetObjectPath(event);
         break;
      case Constants.FATAL_EVENT:
         eventSeverity = RAIDEventProvider.CRITICAL_EVENT;
         targetObjectPath = getTargetObjectPath(event);
         break;
      case Constants.INFORMATION_EVENT:
         eventSeverity = RAIDEventProvider.NORMAL_EVENT;
         targetObjectPath = getTargetObjectPath(event);
         break;
      case Constants.PROGRESS_EVENT:
         eventSeverity = RAIDEventProvider.NORMAL_EVENT;
         targetObjectPath = getTargetObjectPath(event);
         break;
      default:
      };            

      // For now the target will always be the controller since there is currently no easy way
      // to associate an event and object.
      indication.setProperty("AlertingManagedElement", new CIMValue(targetObjectPath));
      indication.setProperty("Description", new CIMValue(event.getEventString()));      
      indication.setProperty("IndicationTime", new CIMValue(new CIMDateTime()));            
      indication.setProperty("AlertType", new CIMValue(new UnsignedInt16(5) /* Device Alert */));            
      indication.setProperty("PerceivedSeverity", new CIMValue(new UnsignedInt16(eventSeverity) /* Device Alert */));
      indication.setProperty("Trending", new CIMValue(new UnsignedInt16(0) /* Unknown */));
      // FixMe!
      //indication.setProperty("ProbablyCause", new CIMValue(new UnsignedInt16(?)));      

      // FixMe!
      //indication.setProperty("IndicationIdentifier", new CIMValue(new UnsignedInt16(?)));
      
      return indication;
   }

   /**
     * This method associates an appropiate CIM object with an event.  For example a defunct drive event is associated
     * with a disk drive, and a critical logical drive event is associated with the appropiate logical drive.
     * @param evt the RaidEvent object received from the ServeRAID Manager agent. 
     # @return String representing the associated object's ObjectPath
     */ 
   String getTargetObjectPath(RaidEvent evt)
   {
      int eventCode = evt.getEventCode();

      String systemCreationClassName = ProviderUtil.getSystemCreationClassName();
      String systemName = JCRMNet.getHostname();
      String creationClassName = null;
      String deviceID = null;      

      // Logical Drive Events
      if ((eventCode > 300) && (eventCode < 321))
      {
         creationClassName = LogicalDrive.CIM_CREATION_CLASS_NAME;
         deviceID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":LogicalDriveID:" + evt.getLogicalDriveID() + ":LogicalDrive");
      }
      // Spanned Array Events
      else if ((eventCode > 321) && (eventCode < 331))
      {
         creationClassName = SpannedArray.CIM_CREATION_CLASS_NAME;
         String collectionID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":SpannedArrayID:" + evt.getArrayID() + ":SpannedArray");
         return creationClassName + ".CreationClassName=\"" + creationClassName + "\"" + ",SystemCreationClassName=\"" + systemCreationClassName + "\"" + ",SystemName=\"" + systemName + "\"" + ",CollectionID=\"" + collectionID +  "\"";
      }
      // More Logical Drive Events
      else if ((eventCode > 330) && (eventCode < 400))
      {
         creationClassName = LogicalDrive.CIM_CREATION_CLASS_NAME;
         deviceID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":LogicalDriveID:" + evt.getLogicalDriveID() + ":LogicalDrive");
      }
      // Disk Drive Events
      else if ((eventCode > 400) && (eventCode < 500))
      {
         creationClassName = HardDrive.CIM_CREATION_CLASS_NAME;
         deviceID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":ChannelID:" + evt.getChannelID() + ":DiskDriveID:" + evt.getSCSIID() + ":DiskDrive");
      }
      // Enclosure Events
      else if ((eventCode > 500) && (eventCode < 600))
      {
         creationClassName = EnclosureDevice.CIM_CREATION_CLASS_NAME;
         deviceID = new String("IBM:ServeRAID:ControllerID:" + evt.getAdapterNumber() + ":ChannelID:" + evt.getChannelID() + ":EnclosureID:" + evt.getSCSIID() + ":Enclosure");
      }
      else
         // Default to the Controller
      {
         creationClassName = Adapter.CIM_CREATION_CLASS_NAME;
         Adapter adapter = null;

         try
         {
            adapter = getRaidSystem().getAdapter(Adapter.adjustedIDToAdapterID(evt.getAdapterNumber()));
         }
         catch (RemoteException e)
         {     
            return Adapter.CIM_CREATION_CLASS_NAME + ".CreationClassName=\"" + Adapter.CIM_CREATION_CLASS_NAME + "\"" + ",SystemCreationClassName=\"" + systemCreationClassName + "\"" + ",SystemName=\"" + systemName + "\"" + ",DeviceID=\"" + (evt.getAdapterNumber() - 1)+  "\"";
         }         

         if (adapter == null)
         {
            eventLog.println("!!! Unable to resolve ObjectPath for eventCode " + eventCode + ", getAdapter() returned NULL");
            return "";
         }

         if (adapter instanceof ServeRaidAdapter)
            creationClassName = ServeRaidAdapter.CIM_CREATION_CLASS_NAME;
         else if (adapter instanceof LSIAdapter)
            creationClassName = LSIAdapter.CIM_CREATION_CLASS_NAME;

         deviceID = new String("IBM:ServeRAID:ControllerID:" + adapter.getDisplayID());
      }

      return creationClassName + ".CreationClassName=\"" + creationClassName + "\"" + ",SystemCreationClassName=\"" + systemCreationClassName + "\"" + ",SystemName=\"" + systemName + "\"" + ",DeviceID=\"" + deviceID +  "\"";
   }

   /**
     * This method processes the alerts received from the ServeRAID Manager agent. 
     * @param evt the RaidEvent object received from the ServeRAID Manager agent. 
     */ 
   public void processAlert(RaidEvent evt)
   {    
      eventLog.println();
      eventLog.println(evt);      
      sendEvent(evt, RAID_EVENT);
   }
   
   public static void main(String[] args)
   {
      RAIDEventProvider provider = new RAIDEventProvider();

      try
      {
         provider.activateFilter(null, RAID_EVENT, null, true);
      }
      catch (CIMException e)
      {
         e.printStackTrace();
         return;
      }
   }
}
