package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocBasicLogicalDrive.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/05/02

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;

/** Represents a Basic Logical Drive on an IROC Adapter.*/
public class IrocBasicLogicalDrive extends BasicLogicalDrive
{
   static final long serialVersionUID = -4912302965880275897L;

   /**
     * Creates a BasicLogicalDrive object with the given parameters.
     * @param  adapterIN                The adapter who owns this HLD
     * @param  arrayIN                  The array to which this HLD belongs
     * @param  logicalDriveIDIN         The logical drive ID of this HLD
     * @param  stateIN                  State of the logical drive
     * @param  raidLevelIN              Raid Level of this HLD
     * @param  dataSpaceIN              Amount of datay space in Bytes
     * @param  paritySpaceIN            Amount of parity space in Bytes
     * @param  blockedAccessIN          True if this logical drive is blocked
     * @param  writeCacheModeIN         Write cache mode
     * @param  lastDriveInArray         True if this drive is the last in its array
     * @see    com.ibm.sysmgt.raidmgr.dataproc.BasicLogicalDrive
     * @see    com.ibm.sysmgt.raidmgr.dataproc.LogicalDrive
     */
   public IrocBasicLogicalDrive(Adapter adapterIN, 
                                Array arrayIN, 
                                int logicalDriveIDIN, 
                                int stateIN, 
                                int raidLevelIN,
                                int dataSpaceIN, 
                                int paritySpaceIN, 
                                boolean blockedAccessIN,
                                int writeCacheModeIN,
                                boolean lastDriveInArrayIN)
   {
      super(adapterIN, 
            arrayIN, 
            logicalDriveIDIN, 
            stateIN, 
            raidLevelIN, 
            dataSpaceIN, 
            paritySpaceIN, 
            "",                  // date created not available 
            blockedAccessIN, 
            writeCacheModeIN, 
            -1,                  // merge group not supported
            false,               // shared drives not supported
            true,                // drive exists
            false,               // temp write through not supported
            lastDriveInArrayIN,
            null);               // state flags
   }

   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      if (getCachedActions() == null)
      {
         Vector actions = new Vector();

         actions.addElement(new HelpNowAction(this));
         actions.addElement(new NullAction());
         //actions.addElement(new InitLogDriveAction(this));
         //actions.addElement(new SyncLogDriveAction(this));
         actions.addElement(new DeleteLogDriveAction(this, false /*compositeAction*/));
         actions.addElement(new NullAction());
         actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_LOG_DRV, getArrayID(), getID()));
         actions.addElement(new PropertiesAction(this));
      
         setActions(actions);
      }

      return getCachedActions();
   }

   /** Returns true if some operation on this logical drive has caused all actions
     * to be blocked.
     * @return true if all actions should be blocked in the user interface
     */
   public boolean shouldBlockAllActions()
   {
      return hasProgress();
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public RaidObjectPropertySet getRaidObjectPropertySet()
   {
      RaidObjectPropertySet set = new RaidObjectPropertySet(getDisplayName(), "l_logdr.gif");
      RaidObjectPropertyGroup group = new RaidObjectPropertyGroup(getShortDisplayName());
      set.addGroup(group);
      set.setDisplayDataSetNames(getDisplayDataSetNames());

      String stateString;
      switch (getState())
      {
         case OFL:  stateString = JCRMUtil.getNLSString("infoLogDriveOffline");     break;
         case OKY:  stateString = JCRMUtil.getNLSString("infoLogDriveOkay");        break;
         case CRT:  stateString = JCRMUtil.getNLSString("infoLogDriveCritical");    break;
         default:   stateString = JCRMUtil.getNLSString("unknown"); 
      }

      if (isBlocked())
         stateString = JCRMUtil.getNLSString("infoLogDriveBlocked");

      group.addElement(new Object[] { new ToolTipString("infoLogDriveNum"), getDisplayID()});
      group.addElement(new Object[] { new ToolTipString("infoArrayLetter"), Array.IDToLetter(getArrayID())});
      group.addElement(new Object[] { new ToolTipString("infoLogDriveState"), new StatusString(stateString, getOverallStatus())});
      group.addElement(new Object[] { new ToolTipString("infoLogDriveRaid"), getDisplayRaidLevel()});
      group.addElement(new Object[] { new ToolTipString("infoLogDriveData"), String.valueOf(getDataSpace()) + " " + JCRMUtil.getDisplayUnits()});
      group.addElement(new Object[] { new ToolTipString("infoLogDriveParity"), String.valueOf(getParitySpace()) + " " + JCRMUtil.getDisplayUnits()});
      group.addElement(new Object[] { new ToolTipString("infoLogDriveSUS"), String.valueOf(getArray().getStripeSize()) + 'K'});

      return set;
   }

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      return "helpIROCLogicalDrive";
   }

   public Object clone()
   {
      IrocBasicLogicalDrive clone = (IrocBasicLogicalDrive) super.clone();
      return clone;
   }
}
